using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.CompilerServices;

public class CharacterController : MonoBehaviour
{
    public AudioClip JumpSound;
    public AudioClip TowerIncreaseSound;
    public AudioClip LoseSound;

    public GameObject MainCharacter = null;
    public bool IsMainCharacter = false;
    public float JumpPower;
    public float Speed = 3f;
    public bool IsPartOfTower = false;

    private bool _isGrounded = false;
    private GameManager _gameManager;

    void Start()
	{
        _gameManager = FindObjectOfType<GameManager>();
		animation.Play("Idle");
	}
	
	void Update()
	{
        if (!_gameManager.GameHasStarted)
		{
			return;
		}
		else
		{
            if(IsMainCharacter == true)
            {
                if (!animation.IsPlaying("Run") & !animation.IsPlaying("Jump"))
                {
                    animation.Play("Run");
                }
                else if (Input.GetMouseButtonDown(0) & animation.IsPlaying("Run") & _isGrounded)
                {
                    rigidbody.linearVelocity = Vector3.zero;
                    rigidbody.angularVelocity = Vector3.zero;
                    rigidbody.linearVelocity = rigidbody.linearVelocity + Vector3.up * JumpPower;
                    PlaySound(JumpSound);
                }

                if (_isGrounded == false & !animation.IsPlaying("Jump"))
                {
                    animation.Play("Jump");
                }
                else if (_isGrounded == true & !animation.IsPlaying("Run"))
                {
                    animation.Play("Run");
                }

                CharacterController[] characters = FindObjectsOfType<CharacterController>();
                foreach (CharacterController character in characters)
                {
                    if (character != this & character.IsPartOfTower == false)
                    {
                        if (_isGrounded == false)
                        {
                            if (Vector3.Distance(transform.position, character.transform.position) < 2)
                            {
                                if ((transform.position.y - character.transform.position.y) > 1.5f)
                                {
                                    SetMainCharacter(character);
                                }
                                else
                                {
                                    PushCharacter(character);
                                }
                            }
                        }
                        else
                        {
                            if (Vector3.Distance(transform.position, character.transform.position) < 1)
                            {

                                PushCharacter(character);

                            }
                        }

                    }
                }

                Wall[] walls = FindObjectsOfType<Wall>();  

                foreach (Wall wall in walls)
                {

                    if (Vector3.Distance(transform.position, wall.transform.position) < 1.5f & transform.position.z < wall.transform.position.z)
                    {
                        if ((transform.position.y - wall.transform.position.y) < 1f)
                        {
                            CharacterController[] chars = FindObjectsOfType<CharacterController>();
                            foreach (CharacterController character in chars)
                            {
                                if (character.IsMainCharacter | character.IsPartOfTower)
                                {
                                    character.animation.Play("FallBackwards");
                                    MainCharacter = null;
                                    character.rigidbody.enabled = true;
                                    character.GetComponent<CapsuleCollider>().enabled = true;
                                    character.enabled = false;
                                    _gameManager.GameOver();
                                    PlaySound(LoseSound);

                                }


                            }
                        }
                        
                    }
                }

                transform.position += Vector3.forward * Speed * Time.deltaTime;
            }
            else
            {
                if (!animation.IsPlaying("Idle"))
                {
                    animation.Play("Idle");
                }
                if (MainCharacter != null)
                {
                    transform.position = MainCharacter.transform.position + Vector3.up * 1.5f;
                }
            }
        }
       CheckIsGrounded();
    }

    void CheckIsGrounded()
	{
        if (transform.position.y < 1.3f)
        {
            _isGrounded = true;
        }
        else
        {
            _isGrounded = false;
        }
    }
    void SetMainCharacter(CharacterController character)
    {
        MainCharacter = character.gameObject;
        IsPartOfTower = true;
        IsMainCharacter = false;
        rigidbody.enabled = false;
        GetComponent<CapsuleCollider>().enabled = false;
        character.IsMainCharacter = true;
        _gameManager.IncreaseScore(50);
        PlaySound(TowerIncreaseSound);
    }

    void PushCharacter(CharacterController character)
    {
        character.animation.Play("Fall");
        character.rigidbody.enabled = false;
        character.GetComponent<CapsuleCollider>().enabled = false;
        Destroy(character);
    }

    public void PlaySound(AudioClip sound)
    {
        audioSource.audioClip = sound;
        audioSource.Play();
    }
}
